package model

import (
	md52 "github.com/fonchain_enterprise/fonchain-approval/pkg/common/md5"
	"gorm.io/plugin/soft_delete"
	"math/rand"
	"strconv"
	"time"
)

// ApprovalType 审批类型
type ApprovalType struct {
	ID        uint64                `gorm:"primaryKey;column:id" json:"id"`     // ID
	DeletedAt soft_delete.DeletedAt `gorm:"column:deleted_at" json:"deletedAt"` // 删除时间
	CreatedAt time.Time             `gorm:"column:created_at" json:"createdAt"` // 创建时间
	UpdatedAt time.Time             `gorm:"column:updated_at" json:"updatedAt"` // 更新时间
	KeyWord   string                `gorm:"column:key_word" json:"keyWord"`     // 回复内容
	Domain    string                `gorm:"column:domain" json:"domain"`        // 回复内容
	Title     string                `gorm:"column:title" json:"title"`          // 回复内容
	Remark    string                `gorm:"column:remark" json:"remark"`        // 回复内容
}

// TableName get sql table name.获取数据库表名
func (m *ApprovalType) TableName() string {
	return "approval_type"
}

var keyWordMap = map[string]string{
	"普通审批":    "content",
	"画作申请审批":  "work",
	"画展包审批":   "exhibition",
	"画展包数量审批": "bundle",
	"画展审批":    "show",
}

func DataExist(domain, keyWork string) bool {
	var count int64
	DB.Model(&ApprovalType{}).Where(&ApprovalType{Domain: domain, KeyWord: keyWork}).Count(&count)
	if count != 0 {
		return true

	}
	return false

}

func KeyWordFromTitle(title string) string {
	keyWord := md52.Md5Part(title+strconv.Itoa(rand.Intn(100)), 8, 16)

	if IsSystemKeyWork(title) == true {
		keyWord = keyWordMap[title]
	}

	return keyWord

}

func IsSystemKeyWork(title string) bool {
	_, isOk := keyWordMap[title]
	return isOk
}
